==> cloudpos-common-classes-1.7.32.jar: com/floreantpos/model/util/InvoiceUtil.java <==
package com.floreantpos.model.util;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;

import com.floreantpos.PosLog;
import com.floreantpos.model.MenuItem;
import com.floreantpos.model.OrderHistory;
import com.floreantpos.model.Ticket;
import com.floreantpos.model.TicketItem;
import com.floreantpos.util.CurrencyUtil;
import com.floreantpos.util.NumberUtil;
import com.stripe.model.Charge;
import com.stripe.model.Charge.PaymentMethodDetails;
import com.stripe.model.Charge.PaymentMethodDetails.Card;
import com.stripe.model.Invoice;
import com.stripe.model.Invoice.DiscountAmount;
import com.stripe.model.InvoiceLineItem;

public class InvoiceUtil {

	public static Map<String, Object> buildStripeInvoicePaymentContextMap(Invoice invoice, Charge charge, Ticket ticket) {
		Map<String, Object> invoiceMap = new HashMap<String, Object>();

		invoiceMap.put("invoiceNo", invoice.getId()); //$NON-NLS-1$
		invoiceMap.put("issueDate", DateUtil.formatDate(new Date((charge == null ? invoice.getCreated() : charge.getCreated()) * 1000L))); //$NON-NLS-1$
		invoiceMap.put("customerName", invoice.getCustomerName()); //$NON-NLS-1$
		invoiceMap.put("customerEmail", invoice.getCustomerEmail()); //$NON-NLS-1$
		invoiceMap.put("customerPhone", ticket.getCustomerMobileNo()); //$NON-NLS-1$1

		String currencySymbol = "$"; //$NON-NLS-1$1
		double chargeAmount = (charge == null ? invoice.getAmountPaid().doubleValue() : charge.getAmount().doubleValue()) / 100D;

		invoiceMap.put("cardPayment", true); //$NON-NLS-1$
		invoiceMap.put("cardAmountNumber", NumberUtil.round(chargeAmount)); //$NON-NLS-1$
		invoiceMap.put("cardAmount", currencySymbol + NumberUtil.formatNumber(chargeAmount)); //$NON-NLS-1$
		invoiceMap.put("paidAmount", currencySymbol + NumberUtil.formatNumber(chargeAmount)); //$NON-NLS-1$
		invoiceMap.put("paymentInformation", getCardPaymentInformation(charge)); //$NON-NLS-1$
		invoiceMap.put("paymentStatus", "Paid"); //$NON-NLS-1$ //$NON-NLS-2$
		invoiceMap.put("paymentType", "Credit card"); //$NON-NLS-1$ //$NON-NLS-2$
		invoiceMap.put("orderItemsFromTemplate", getItemsInformation(invoice, ticket, currencySymbol)); //$NON-NLS-1$
		invoiceMap.put("paid", invoice.getPaid()); //$NON-NLS-1$

		double totalAmount = invoice.getTotal().doubleValue() / 100D;
		double paidAmount = invoice.getPaid() ? totalAmount : (invoice.getAmountPaid().doubleValue() / 100D);
		double discountAmount = 0D;
		List<DiscountAmount> discountAmounts = invoice.getTotalDiscountAmounts();
		if (discountAmounts != null && discountAmounts.size() > 0) {
			for (DiscountAmount dAmount : discountAmounts) {
				if (dAmount.getAmount() == null) {
					continue;
				}
				double amount = dAmount.getAmount().doubleValue() / 100;
				discountAmount += amount;
			}
		}
		double dueAmount = invoice.getPaid() ? 0D : (totalAmount - paidAmount);
		invoiceMap.put("dueAmount", currencySymbol + NumberUtil.format(dueAmount)); //$NON-NLS-1$
		invoiceMap.put("subTotal", currencySymbol + NumberUtil.format(totalAmount + discountAmount)); //$NON-NLS-1$
		invoiceMap.put("discountAmount", currencySymbol + NumberUtil.format(discountAmount)); //$NON-NLS-1$
		invoiceMap.put("total", currencySymbol + NumberUtil.format(totalAmount)); //$NON-NLS-1$
		invoiceMap.put("hasDiscount", discountAmount > 0); //$NON-NLS-1$
		invoiceMap.put("amountDue", currencySymbol + NumberUtil.format(dueAmount)); //$NON-NLS-1$
		return invoiceMap;
	}

	public static Map<String, Object> buildWireTransferInvoiceContextMap(OrderHistory orderHistory, Ticket ticket) {
		String invoiceNo = ticket.getProperty("invoiceNo"); //$NON-NLS-1$
		if (orderHistory != null && StringUtils.isBlank(invoiceNo)) {
			return null;
		}
		Map<String, Object> invoiceMap = new HashMap<String, Object>();
		String currencySymbol = CurrencyUtil.getCurrencySymbol();
		invoiceMap.put("invoiceNo", invoiceNo); //$NON-NLS-1$
		String paymentDate = orderHistory.getProperty("paymentDate"); //$NON-NLS-1$
		if (StringUtils.isNotBlank(paymentDate)) {
			invoiceMap.put("issueDate", paymentDate); //$NON-NLS-1$
		}
		else {
			invoiceMap.put("issueDate", DateUtil.formatDate(ticket.getCreateDate())); //$NON-NLS-1$
		}
		invoiceMap.put("customerName", ticket.getCustomerName()); //$NON-NLS-1$
		invoiceMap.put("customerEmail", ticket.getCustomerEmail()); //$NON-NLS-1$
		invoiceMap.put("customerPhone", ticket.getCustomerMobileNo()); //$NON-NLS-1$4
		invoiceMap.put("dueAmount", currencySymbol + NumberUtil.format(ticket.getDueAmount())); //$NON-NLS-1$
		invoiceMap.put("bankPayment", true); //$NON-NLS-1$

		String paymentInfo = getBankPaymentInformation(orderHistory);
		if (StringUtils.isNotBlank(paymentInfo)) {
			invoiceMap.put("paymentInformation", paymentInfo); //$NON-NLS-1$
		}
		invoiceMap.put("paid", ticket.isPaid()); //$NON-NLS-1$
		invoiceMap.put("paidAmount", currencySymbol + NumberUtil.format(ticket.getPaidAmount())); //$NON-NLS-1$
		invoiceMap.put("orderItemsFromTemplate", getItemsInformation(ticket, currencySymbol)); //$NON-NLS-1$
		invoiceMap.put("subTotal", currencySymbol + NumberUtil.format(ticket.getSubtotalAmount())); //$NON-NLS-1$
		invoiceMap.put("discountAmount", currencySymbol + NumberUtil.format(ticket.getDiscountAmount())); //$NON-NLS-1$
		invoiceMap.put("total", currencySymbol + NumberUtil.format(ticket.getTotalAmount())); //$NON-NLS-1$
		invoiceMap.put("amountDue", currencySymbol + NumberUtil.format(ticket.getDueAmount())); //$NON-NLS-1$
		invoiceMap.put("hasDue", ticket.getDueAmount() > 0); //$NON-NLS-1$
		invoiceMap.put("hasDiscount", ticket.getDiscountAmount() > 0); //$NON-NLS-1$
		return invoiceMap;
	}

	public static String getItemsInformation(Invoice invoice, Ticket ticket, String currencySymbol) {
		if (invoice == null || invoice.getLines() == null) {
			return ""; //$NON-NLS-1$
		}
		Map<String, String> itemNameMap = new HashMap<>();
		for (TicketItem ticketItem : ticket.getTicketItems()) {
			MenuItem menuItem = ticketItem.getMenuItem();
			if (menuItem != null && menuItem.isVariant() && menuItem.getParentMenuItemId() != null) {
				itemNameMap.put(menuItem.getParentMenuItemId(), ticketItem.getNameDisplay(ticketItem.getName(), false, false, false));
				continue;
			}
			itemNameMap.put(ticketItem.getMenuItemId(), ticketItem.getNameDisplay(ticketItem.getName(), false, false, false));
		}
		List<InvoiceLineItem> orderItems = invoice.getLines().getData();
		final StringBuilder itemsBuilder = new StringBuilder(""); //$NON-NLS-1$
		if (orderItems != null && orderItems.size() > 0) {
			int count = 1;
			for (InvoiceLineItem orderItem : orderItems) {
				double quantity = orderItem.getQuantity().doubleValue();
				if (quantity <= 0) {
					continue;
				}
				double totalAmount = orderItem.getAmount().doubleValue() / 100D;
				if (totalAmount <= 0) {
					continue;
				}
				StringBuilder itemRow = new StringBuilder("<tr style=\"border-right:none !important;border-left: none !important;\">"); //$NON-NLS-1$
				itemRow.append("<td style=\"width: 70px;border-right: none !important;border-left: none !important;\">").append(String.valueOf(count)) //$NON-NLS-1$
						.append("</td>"); //$NON-NLS-1$
				String productName = orderItem.getDescription();
				if (orderItem.getPrice() != null) {
					String name = itemNameMap.get(orderItem.getPrice().getProduct());
					if (StringUtils.isNotBlank(name)) {
						productName = name;
					}
				}
				double subtotalAmount = totalAmount / quantity;
				itemRow.append("<td style=\"width: 358px;border-right: none !important;border-left: none !important;\">") //$NON-NLS-1$
						.append(productName).append("</td>"); //$NON-NLS-1$
				itemRow.append("<td style=\"width: 100px;border-right: none !important;border-left: none !important;\">") //$NON-NLS-1$
						.append(NumberUtil.formatNumberIfNeeded(quantity)).append("</td>"); //$NON-NLS-1$
				itemRow.append("<td style=\"width: 120px;text-align: right;border-right: none !important;border-left: none !important;\">") //$NON-NLS-1$
						.append(currencySymbol + NumberUtil.format(subtotalAmount)).append("</td>"); //$NON-NLS-1$
				itemRow.append("<td style=\"width: 200px;text-align: right;border-right: none !important;border-left: none !important;\">") //$NON-NLS-1$
						.append(currencySymbol + NumberUtil.format(totalAmount)).append("</td>"); //$NON-NLS-1$
				itemRow.append("</tr>"); //$NON-NLS-1$

				itemsBuilder.append(itemRow.toString());
				count++;
			}
		}
		return itemsBuilder.toString();
	}

	public static String getItemsInformation(Ticket ticket, String currencySymbol) {
		List<TicketItem> orderItems = ticket.getTicketItems();
		final StringBuilder itemsBuilder = new StringBuilder(""); //$NON-NLS-1$
		if (orderItems != null && orderItems.size() > 0) {
			int count = 1;
			for (TicketItem orderItem : orderItems) {
				StringBuilder itemRow = new StringBuilder("<tr style=\"border-right:none !important;border-left: none !important;\">"); //$NON-NLS-1$
				itemRow.append("<td style=\"width: 70px;border-right: none !important;border-left: none !important;\">").append(String.valueOf(count)) //$NON-NLS-1$
						.append("</td>"); //$NON-NLS-1$
				itemRow.append("<td style=\"width: 358px;border-right: none !important;border-left: none !important;\">") //$NON-NLS-1$
						.append(orderItem.getNameDisplay(orderItem.getName(), false, false, false)).append("</td>"); //$NON-NLS-1$
				itemRow.append("<td style=\"width: 100px;border-right: none !important;border-left: none !important;\">") //$NON-NLS-1$
						.append(NumberUtil.formatNumberIfNeeded(orderItem.getQuantity())).append("</td>"); //$NON-NLS-1$
				itemRow.append("<td style=\"width: 120px;text-align: right;border-right: none !important;border-left: none !important;\">") //$NON-NLS-1$
						.append(currencySymbol + NumberUtil.format(orderItem.getUnitPrice())).append("</td>"); //$NON-NLS-1$
				itemRow.append("<td style=\"width: 200px;text-align: right;border-right: none !important;border-left: none !important;\">") //$NON-NLS-1$
						.append(currencySymbol + NumberUtil.format(orderItem.getTotalAmount())).append("</td>"); //$NON-NLS-1$
				itemRow.append("</tr>"); //$NON-NLS-1$

				itemsBuilder.append(itemRow.toString());
				count++;
			}
		}
		return itemsBuilder.toString();
	}

	private static String getBankPaymentInformation(OrderHistory orderHistory) {
		if (orderHistory == null) {
			return null;
		}
		String br = "<br>"; //$NON-NLS-1$
		String paymentInfo = ""; //$NON-NLS-1$
		try {
			String paymentDate = orderHistory.getProperty("paymentDate"); //$NON-NLS-1$
			if (StringUtils.isNotBlank(paymentDate)) {
				paymentInfo += br + "Payment date: " + paymentDate; //$NON-NLS-1$
			}
			String reference = orderHistory.getProperty("reference"); //$NON-NLS-1$
			if (StringUtils.isNotBlank(reference)) {
				paymentInfo += br + "Reference: " + reference; //$NON-NLS-1$
			}
			String notes = orderHistory.getProperty("notes"); //$NON-NLS-1$
			if (StringUtils.isNotBlank(notes)) {
				paymentInfo += br + "Notes: " + notes; //$NON-NLS-1$
			}
			if (StringUtils.isBlank(paymentInfo)) {
				return null;
			}
		} catch (Exception e) {
			PosLog.error(InvoiceUtil.class, e);
		}
		return "<b>Wire transfer information:</b>" + paymentInfo; //$NON-NLS-1$
	}

	private static String getCardPaymentInformation(Charge transaction) {
		if (transaction == null) {
			return "<br><br>"; //$NON-NLS-1$
		}
		String br = "<br>"; //$NON-NLS-1$
		String string = "<br><br><b>Payment information:</b>"; //$NON-NLS-1$
		try {
			PaymentMethodDetails paymentMethodDetails = transaction.getPaymentMethodDetails();
			Card card = paymentMethodDetails.getCard();
			string += br + "Card type: " + card.getBrand(); //$NON-NLS-1$
			String cardNumber = card.getLast4();
			if (cardNumber != null) {
				string += br + "Acct: " + "*** *** ***" + cardNumber; //$NON-NLS-1$ //$NON-NLS-2$
			}
			if (card.getIssuer() != null) {
				string += br + "Card holder: " + card.getIssuer(); //$NON-NLS-1$
			}
			if (transaction.getId() != null) {
				string += br + "Trans id: " + transaction.getId(); //$NON-NLS-1$
			}
			if (transaction.getAuthorizationCode() != null) {
				string += br + "Approval: " + transaction.getAuthorizationCode(); //$NON-NLS-1$
			}
		} catch (Exception e) {
			PosLog.error(InvoiceUtil.class, e);
		}
		return string;
	}
}

==> cloudpos-common-classes-1.7.33-20230412.052846-25.jar: com/floreantpos/model/util/InvoiceUtil.java <==
package com.floreantpos.model.util;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;

import com.floreantpos.PosLog;
import com.floreantpos.model.MenuItem;
import com.floreantpos.model.OrderHistory;
import com.floreantpos.model.Ticket;
import com.floreantpos.model.TicketItem;
import com.floreantpos.util.CurrencyUtil;
import com.floreantpos.util.NumberUtil;
import com.stripe.model.Charge;
import com.stripe.model.Charge.PaymentMethodDetails;
import com.stripe.model.Charge.PaymentMethodDetails.Card;
import com.stripe.model.Invoice;
import com.stripe.model.Invoice.DiscountAmount;
import com.stripe.model.InvoiceLineItem;

public class InvoiceUtil {

	public static Map<String, Object> buildStripeInvoicePaymentContextMap(Invoice invoice, Charge charge, Ticket ticket) {
		Map<String, Object> invoiceMap = new HashMap<String, Object>();

		invoiceMap.put("invoiceNo", invoice.getId()); //$NON-NLS-1$
		invoiceMap.put("issueDate", DateUtil.formatDate(new Date((charge == null ? invoice.getCreated() : charge.getCreated()) * 1000L))); //$NON-NLS-1$
		invoiceMap.put("customerName", invoice.getCustomerName()); //$NON-NLS-1$
		invoiceMap.put("customerEmail", invoice.getCustomerEmail()); //$NON-NLS-1$
		invoiceMap.put("customerPhone", ticket.getCustomerMobileNo()); //$NON-NLS-1$1

		String currencySymbol = "$"; //$NON-NLS-1$1
		double chargeAmount = (charge == null ? invoice.getAmountPaid().doubleValue() : charge.getAmount().doubleValue()) / 100D;

		invoiceMap.put("cardPayment", true); //$NON-NLS-1$
		invoiceMap.put("cardAmountNumber", NumberUtil.round(chargeAmount)); //$NON-NLS-1$
		invoiceMap.put("cardAmount", currencySymbol + NumberUtil.formatNumber(chargeAmount)); //$NON-NLS-1$
		invoiceMap.put("paidAmount", currencySymbol + NumberUtil.formatNumber(chargeAmount)); //$NON-NLS-1$
		invoiceMap.put("paymentInformation", getCardPaymentInformation(charge)); //$NON-NLS-1$
		invoiceMap.put("paymentStatus", "Paid"); //$NON-NLS-1$ //$NON-NLS-2$
		invoiceMap.put("paymentType", "Credit card"); //$NON-NLS-1$ //$NON-NLS-2$
		invoiceMap.put("orderItemsFromTemplate", getItemsInformation(invoice, ticket, currencySymbol)); //$NON-NLS-1$
		invoiceMap.put("paid", invoice.getPaid()); //$NON-NLS-1$

		double totalAmount = invoice.getTotal().doubleValue() / 100D;
		double paidAmount = invoice.getPaid() ? totalAmount : (invoice.getAmountPaid().doubleValue() / 100D);
		double discountAmount = 0D;
		List<DiscountAmount> discountAmounts = invoice.getTotalDiscountAmounts();
		if (discountAmounts != null && discountAmounts.size() > 0) {
			for (DiscountAmount dAmount : discountAmounts) {
				if (dAmount.getAmount() == null) {
					continue;
				}
				double amount = dAmount.getAmount().doubleValue() / 100;
				discountAmount += amount;
			}
		}
		double dueAmount = invoice.getPaid() ? 0D : (totalAmount - paidAmount);
		invoiceMap.put("dueAmount", currencySymbol + NumberUtil.format(dueAmount)); //$NON-NLS-1$
		invoiceMap.put("subTotal", currencySymbol + NumberUtil.format(totalAmount + discountAmount)); //$NON-NLS-1$
		invoiceMap.put("discountAmount", currencySymbol + NumberUtil.format(discountAmount)); //$NON-NLS-1$
		invoiceMap.put("total", currencySymbol + NumberUtil.format(totalAmount)); //$NON-NLS-1$
		invoiceMap.put("hasDiscount", discountAmount > 0); //$NON-NLS-1$
		invoiceMap.put("amountDue", currencySymbol + NumberUtil.format(dueAmount)); //$NON-NLS-1$
		return invoiceMap;
	}

	public static Map<String, Object> buildWireTransferInvoiceContextMap(OrderHistory orderHistory, Ticket ticket) {
		String invoiceNo = ticket.getProperty("invoiceNo"); //$NON-NLS-1$
		if (orderHistory != null && StringUtils.isBlank(invoiceNo)) {
			return null;
		}
		Map<String, Object> invoiceMap = new HashMap<String, Object>();
		String currencySymbol = CurrencyUtil.getCurrencySymbol();
		invoiceMap.put("invoiceNo", invoiceNo); //$NON-NLS-1$
		String paymentDate = orderHistory.getProperty("paymentDate"); //$NON-NLS-1$
		if (StringUtils.isNotBlank(paymentDate)) {
			invoiceMap.put("issueDate", paymentDate); //$NON-NLS-1$
		}
		else {
			invoiceMap.put("issueDate", DateUtil.formatDate(ticket.getCreateDate())); //$NON-NLS-1$
		}
		invoiceMap.put("customerName", ticket.getCustomerName()); //$NON-NLS-1$
		invoiceMap.put("customerEmail", ticket.getCustomerEmail()); //$NON-NLS-1$
		invoiceMap.put("customerPhone", ticket.getCustomerMobileNo()); //$NON-NLS-1$4
		invoiceMap.put("dueAmount", currencySymbol + NumberUtil.format(ticket.getDueAmount())); //$NON-NLS-1$
		invoiceMap.put("bankPayment", true); //$NON-NLS-1$

		String paymentInfo = getBankPaymentInformation(orderHistory);
		if (StringUtils.isNotBlank(paymentInfo)) {
			invoiceMap.put("paymentInformation", paymentInfo); //$NON-NLS-1$
		}
		invoiceMap.put("paid", ticket.isPaid()); //$NON-NLS-1$
		invoiceMap.put("paidAmount", currencySymbol + NumberUtil.format(ticket.getPaidAmount())); //$NON-NLS-1$
		invoiceMap.put("orderItemsFromTemplate", getItemsInformation(ticket, currencySymbol)); //$NON-NLS-1$
		invoiceMap.put("subTotal", currencySymbol + NumberUtil.format(ticket.getSubtotalAmount())); //$NON-NLS-1$
		invoiceMap.put("discountAmount", currencySymbol + NumberUtil.format(ticket.getDiscountAmount())); //$NON-NLS-1$
		invoiceMap.put("total", currencySymbol + NumberUtil.format(ticket.getTotalAmount())); //$NON-NLS-1$
		invoiceMap.put("amountDue", currencySymbol + NumberUtil.format(ticket.getDueAmount())); //$NON-NLS-1$
		invoiceMap.put("hasDue", ticket.getDueAmount() > 0); //$NON-NLS-1$
		invoiceMap.put("hasDiscount", ticket.getDiscountAmount() > 0); //$NON-NLS-1$
		return invoiceMap;
	}

	public static String getItemsInformation(Invoice invoice, Ticket ticket, String currencySymbol) {
		if (invoice == null || invoice.getLines() == null) {
			return ""; //$NON-NLS-1$
		}
		Map<String, String> itemNameMap = new HashMap<>();
		for (TicketItem ticketItem : ticket.getTicketItems()) {
			MenuItem menuItem = ticketItem.getMenuItem();
			String name = ticketItem.getNameDisplay(ticketItem.getName(), false, false, false);
			if (menuItem != null && menuItem.isVariant() && menuItem.getParentMenuItemId() != null) {
				itemNameMap.put(menuItem.getParentMenuItemId(), name);
				continue;
			}
			itemNameMap.put(ticketItem.getMenuItemId(), name);
		}
		List<InvoiceLineItem> orderItems = invoice.getLines().getData();
		final StringBuilder itemsBuilder = new StringBuilder(""); //$NON-NLS-1$
		if (orderItems != null && orderItems.size() > 0) {
			int count = 1;
			for (InvoiceLineItem orderItem : orderItems) {
				double quantity = orderItem.getQuantity().doubleValue();
				if (quantity <= 0) {
					continue;
				}
				double totalAmount = orderItem.getAmount().doubleValue() / 100D;
				if (totalAmount <= 0) {
					continue;
				}
				StringBuilder itemRow = new StringBuilder("<tr style=\"border-right:none !important;border-left: none !important;\">"); //$NON-NLS-1$
				itemRow.append("<td style=\"width: 70px;border-right: none !important;border-left: none !important;\">").append(String.valueOf(count)) //$NON-NLS-1$
						.append("</td>"); //$NON-NLS-1$
				String productName = orderItem.getDescription();
				if (orderItem.getPrice() != null) {
					String name = itemNameMap.get(orderItem.getPrice().getProduct());
					if (StringUtils.isNotBlank(name)) {
						productName = name;
					}
				}
				double subtotalAmount = totalAmount / quantity;
				itemRow.append("<td style=\"width: 358px;border-right: none !important;border-left: none !important;\">") //$NON-NLS-1$
						.append(productName).append("</td>"); //$NON-NLS-1$
				itemRow.append("<td style=\"width: 100px;border-right: none !important;border-left: none !important;\">") //$NON-NLS-1$
						.append(NumberUtil.formatNumberIfNeeded(quantity)).append("</td>"); //$NON-NLS-1$
				itemRow.append("<td style=\"width: 120px;text-align: right;border-right: none !important;border-left: none !important;\">") //$NON-NLS-1$
						.append(currencySymbol + NumberUtil.format(subtotalAmount)).append("</td>"); //$NON-NLS-1$
				itemRow.append("<td style=\"width: 200px;text-align: right;border-right: none !important;border-left: none !important;\">") //$NON-NLS-1$
						.append(currencySymbol + NumberUtil.format(totalAmount)).append("</td>"); //$NON-NLS-1$
				itemRow.append("</tr>"); //$NON-NLS-1$

				itemsBuilder.append(itemRow.toString());
				count++;
			}
		}
		return itemsBuilder.toString();
	}

	public static String getItemsInformation(Ticket ticket, String currencySymbol) {
		List<TicketItem> orderItems = ticket.getTicketItems();
		final StringBuilder itemsBuilder = new StringBuilder(""); //$NON-NLS-1$
		if (orderItems != null && orderItems.size() > 0) {
			int count = 1;
			for (TicketItem orderItem : orderItems) {
				StringBuilder itemRow = new StringBuilder("<tr style=\"border-right:none !important;border-left: none !important;\">"); //$NON-NLS-1$
				itemRow.append("<td style=\"width: 70px;border-right: none !important;border-left: none !important;\">").append(String.valueOf(count)) //$NON-NLS-1$
						.append("</td>"); //$NON-NLS-1$
				itemRow.append("<td style=\"width: 358px;border-right: none !important;border-left: none !important;\">") //$NON-NLS-1$
						.append(orderItem.getNameDisplay(orderItem.getName(), false, false, false)).append("</td>"); //$NON-NLS-1$
				itemRow.append("<td style=\"width: 100px;border-right: none !important;border-left: none !important;\">") //$NON-NLS-1$
						.append(NumberUtil.formatNumberIfNeeded(orderItem.getQuantity())).append("</td>"); //$NON-NLS-1$
				itemRow.append("<td style=\"width: 120px;text-align: right;border-right: none !important;border-left: none !important;\">") //$NON-NLS-1$
						.append(currencySymbol + NumberUtil.format(orderItem.getUnitPrice())).append("</td>"); //$NON-NLS-1$
				itemRow.append("<td style=\"width: 200px;text-align: right;border-right: none !important;border-left: none !important;\">") //$NON-NLS-1$
						.append(currencySymbol + NumberUtil.format(orderItem.getTotalAmount()) + orderItem.getShortInterval()).append("</td>"); //$NON-NLS-1$
				itemRow.append("</tr>"); //$NON-NLS-1$

				itemsBuilder.append(itemRow.toString());
				count++;
			}
		}
		return itemsBuilder.toString();
	}

	private static String getBankPaymentInformation(OrderHistory orderHistory) {
		if (orderHistory == null) {
			return null;
		}
		String br = "<br>"; //$NON-NLS-1$
		String paymentInfo = ""; //$NON-NLS-1$
		try {
			String paymentDate = orderHistory.getProperty("paymentDate"); //$NON-NLS-1$
			if (StringUtils.isNotBlank(paymentDate)) {
				paymentInfo += br + "Payment date: " + paymentDate; //$NON-NLS-1$
			}
			String reference = orderHistory.getProperty("reference"); //$NON-NLS-1$
			if (StringUtils.isNotBlank(reference)) {
				paymentInfo += br + "Reference: " + reference; //$NON-NLS-1$
			}
			String notes = orderHistory.getProperty("notes"); //$NON-NLS-1$
			if (StringUtils.isNotBlank(notes)) {
				paymentInfo += br + "Notes: " + notes; //$NON-NLS-1$
			}
			if (StringUtils.isBlank(paymentInfo)) {
				return null;
			}
		} catch (Exception e) {
			PosLog.error(InvoiceUtil.class, e);
		}
		return "<b>Wire transfer information:</b>" + paymentInfo; //$NON-NLS-1$
	}

	private static String getCardPaymentInformation(Charge transaction) {
		if (transaction == null) {
			return "<br><br>"; //$NON-NLS-1$
		}
		String br = "<br>"; //$NON-NLS-1$
		String string = "<br><br><b>Payment information:</b>"; //$NON-NLS-1$
		try {
			PaymentMethodDetails paymentMethodDetails = transaction.getPaymentMethodDetails();
			Card card = paymentMethodDetails.getCard();
			string += br + "Card type: " + card.getBrand(); //$NON-NLS-1$
			String cardNumber = card.getLast4();
			if (cardNumber != null) {
				string += br + "Acct: " + "*** *** ***" + cardNumber; //$NON-NLS-1$ //$NON-NLS-2$
			}
			if (card.getIssuer() != null) {
				string += br + "Card holder: " + card.getIssuer(); //$NON-NLS-1$
			}
			if (transaction.getId() != null) {
				string += br + "Trans id: " + transaction.getId(); //$NON-NLS-1$
			}
			if (transaction.getAuthorizationCode() != null) {
				string += br + "Approval: " + transaction.getAuthorizationCode(); //$NON-NLS-1$
			}
		} catch (Exception e) {
			PosLog.error(InvoiceUtil.class, e);
		}
		return string;
	}

	public static Map<String, Object> buildInvoiceContextMap(com.floreantpos.model.Invoice invoice, Ticket ticket) {
		Map<String, Object> invoiceMap = new HashMap<String, Object>();
		String currencySymbol = CurrencyUtil.getCurrencySymbol();

		String invoiceString = invoice.getInvoiceNumberDisplayString();
		invoiceMap.put("invoiceNo", invoiceString); //$NON-NLS-1$
		invoiceMap.put("dueDate", invoice.getDueDate() == null ? "" : DateUtil.formatDate(invoice.getDueDate())); //$NON-NLS-1$
		invoiceMap.put("issueDate", DateUtil.formatDate(ticket.getCreateDate())); //$NON-NLS-1$
		invoiceMap.put("customerName", ticket.getCustomerName()); //$NON-NLS-1$
		invoiceMap.put("resellerName", invoice.getProperty("resellerName")); //$NON-NLS-1$
		invoiceMap.put("customerEmail", ticket.getCustomerEmail()); //$NON-NLS-1$
		invoiceMap.put("customerPhone", ticket.getCustomerMobileNo()); //$NON-NLS-1$4
		invoiceMap.put("dueAmount", currencySymbol + NumberUtil.format(ticket.getDueAmount())); //$NON-NLS-1$
		invoiceMap.put("bankPayment", true); //$NON-NLS-1$
		invoiceMap.put("paid", ticket.isPaid()); //$NON-NLS-1$
		invoiceMap.put("paidAmount", currencySymbol + NumberUtil.format(ticket.getPaidAmount())); //$NON-NLS-1$
		invoiceMap.put("orderItemsFromTemplate", getItemsInformation(ticket, currencySymbol)); //$NON-NLS-1$
		invoiceMap.put("subTotal", currencySymbol + NumberUtil.format(ticket.getSubtotalAmount())); //$NON-NLS-1$
		invoiceMap.put("discountAmount", currencySymbol + NumberUtil.format(ticket.getDiscountAmount())); //$NON-NLS-1$
		invoiceMap.put("total", currencySymbol + NumberUtil.format(ticket.getTotalAmount())); //$NON-NLS-1$
		invoiceMap.put("amountDue", currencySymbol + NumberUtil.format(ticket.getDueAmount())); //$NON-NLS-1$
		invoiceMap.put("hasDue", ticket.getDueAmount() > 0); //$NON-NLS-1$
		invoiceMap.put("hasDiscount", ticket.getDiscountAmount() > 0); //$NON-NLS-1$
		return invoiceMap;
	}

}
